@@ -10,12 +10,12 @@ from itertools import product
 from django.apps import apps
 from django.contrib.auth import get_user_model
 from django.core.exceptions import ObjectDoesNotExist
+from django.db.models import Exists, OuterRef
 from django.utils.functional import cached_property
 from django.utils.translation import ugettext as _
 from lms.djangoapps.grades import api as grades_api
 from opaque_keys.edx.keys import CourseKey, UsageKey
 from openedx.core.djangoapps.course_groups.cohorts import get_cohort
-from six import iteritems, text_type
 from super_csv.csv_processor import CSVProcessor, DeferrableMixin, ValidationError
 
 from bulk_grades.clients import LearnerAPIClient
@@ -29,7 +29,7 @@ log = logging.getLogger(__name__)
 UNKNOWN_LAST_SCORE_OVERRIDER = 'unknown'
 
 
-def _get_enrollments(course_id, track=None, cohort=None, active_only=False):
+def _get_enrollments(course_id, track=None, cohort=None, active_only=False, excluded_course_roles=None):
     """
     Return iterator of enrollment dictionaries.
 
@@ -53,6 +53,18 @@ def _get_enrollments(course_id, track=None, cohort=None, active_only=False):
             user__cohortmembership__course_user_group__name=cohort)
     if active_only:
         enrollments = enrollments.filter(is_active=True)
+    if excluded_course_roles:
+        course_access_role_filters = dict(
+            user=OuterRef('user'),
+            course_id=course_id
+        )
+        if 'all' not in excluded_course_roles:
+            course_access_role_filters['role__in'] = excluded_course_roles
+        enrollments = enrollments.annotate(has_excluded_course_role=Exists(
+            apps.get_model('student', 'CourseAccessRole').objects.filter(**course_access_role_filters)
+        ))
+        enrollments = enrollments.exclude(has_excluded_course_role=True)
+
     for enrollment in enrollments:
         enrollment_dict = {
             'user': enrollment.user,
@@ -200,7 +212,7 @@ class ScoreCSVProcessor(DeferrableMixin, CSVProcessor):
             # after commit, trigger grade recomputation for the course.
             # not sure if this is necessary
             course_key = UsageKey.from_string(self.block_id).course_key
-            grades_api.task_compute_all_grades_for_course.apply_async(kwargs={'course_key': text_type(course_key)})
+            grades_api.task_compute_all_grades_for_course.apply_async(kwargs={'course_key': str(course_key)})
 
 
 class GradedSubsectionMixin:
@@ -229,11 +241,11 @@ class GradedSubsectionMixin:
         """
         subsections = OrderedDict()
         for subsection in grades_api.graded_subsections_for_course_id(course_id):
-            block_id = text_type(subsection.location.block_id)
+            block_id = str(subsection.location.block_id)
             if (  # pragma: no branch
                     (filter_subsection and (block_id != filter_subsection.block_id))
                     or
-                    (filter_assignment_type and (filter_assignment_type != text_type(subsection.format)))
+                    (filter_assignment_type and (filter_assignment_type != str(subsection.format)))
             ):
                 continue  # pragma: no cover
             short_block_id = block_id[:8]
@@ -248,7 +260,7 @@ class GradedSubsectionMixin:
         and ``prefixes`` to append to each, returns a list of names
         formed from the product of the subsection ids and prefixes.
         """
-        return ['{}-{}'.format(prefix, short_id) for short_id, prefix in product(short_subsection_ids, prefixes)]
+        return [f'{prefix}-{short_id}' for short_id, prefix in product(short_subsection_ids, prefixes)]
 
 
 def decode_utf8(input_iterator):
@@ -266,7 +278,7 @@ class GradeCSVProcessor(DeferrableMixin, GradedSubsectionMixin, CSVProcessor):
     """
 
     required_columns = ['user_id', 'course_id']
-    subsection_prefixes = ('name', 'original_grade', 'previous_override', 'new_override',)
+    subsection_prefixes = ('name', 'grade', 'original_grade', 'previous_override', 'new_override',)
 
     def __init__(self, **kwargs):
         """
@@ -284,6 +296,7 @@ class GradeCSVProcessor(DeferrableMixin, GradedSubsectionMixin, CSVProcessor):
         self.cohort = None
         self.user_id = None
         self.active_only = False
+        self.excluded_course_roles = None
 
         # The CSVProcessor.__init__ method will set attributes on self
         # from items in kwargs, so this super().__init__() call can
@@ -354,7 +367,7 @@ class GradeCSVProcessor(DeferrableMixin, GradedSubsectionMixin, CSVProcessor):
                     subsection = self._subsections[short_id][0]
                     operation['user_id'] = row['user_id']
                     operation['course_id'] = self.course_id
-                    operation['block_id'] = text_type(subsection.location)
+                    operation['block_id'] = str(subsection.location)
                     try:
                         operation['new_override'] = float(value)
                     except ValueError as error:
@@ -383,7 +396,13 @@ class GradeCSVProcessor(DeferrableMixin, GradedSubsectionMixin, CSVProcessor):
         """
         Return iterator of rows to export.
         """
-        enrollments = list(_get_enrollments(self._course_key, track=self.track, cohort=self.cohort, active_only=self.active_only))
+        enrollments = list(_get_enrollments(
+            self._course_key,
+            track=self.track,
+            cohort=self.cohort,
+            active_only=self.active_only,
+            excluded_course_roles=self.excluded_course_roles,
+        ))
         enrolled_users = [enroll['user'] for enroll in enrollments]
 
         grades_api.prefetch_course_and_subsection_grades(self._course_key, enrolled_users)
@@ -424,17 +443,50 @@ class GradeCSVProcessor(DeferrableMixin, GradedSubsectionMixin, CSVProcessor):
                     (self.course_grade_max and course_grade_normalized > self.course_grade_max)):
                 continue
 
-            for block_id, (subsection, display_name) in iteritems(self._subsections):
-                row['name-{}'.format(block_id)] = display_name
+            for block_id, (subsection, display_name) in self._subsections.items():
+                row[f'name-{block_id}'] = display_name
                 grade = grades.get(subsection.location, None)
                 if grade:
-                    row['original_grade-{}'.format(block_id)] = grade.earned_graded
+                    effective_grade = grade.earned_graded
+                    row[f'original_grade-{block_id}'] = grade.earned_graded
                     try:
-                        row['previous_override-{}'.format(block_id)] = grade.override.earned_graded_override
+                        effective_grade = grade.override.earned_graded_override
+                        row[f'previous_override-{block_id}'] = grade.override.earned_graded_override
                     except AttributeError:
-                        row['previous_override-{}'.format(block_id)] = None
+                        row[f'previous_override-{block_id}'] = None
+                    row[f'grade-{block_id}'] = effective_grade
             yield row
 
+    def filtered_column_headers(self):
+        """
+        To trim down grade exports, only show subsections which were modified in a bulk update.
+        Returns: a filtered list of columns to export, preserving modified and non-subsection columns
+        """
+        unmodified_subsections = set(self._subsections.keys())
+
+        for row in self.result_data:
+            # Ignore rows which didn't introduce changes
+            if row['status'] == 'No Action':
+                continue
+
+            subsections_modified_by_row = set()
+
+            # Get changes this row introduced for an as-yet-unmodified subsection
+            for subsection in unmodified_subsections:
+                if row[f'new_override-{subsection}'].strip() != '':
+                    subsections_modified_by_row.add(subsection)
+
+            # Remove modified subsections from the unmodified list
+            if subsections_modified_by_row:
+                unmodified_subsections.difference_update(subsections_modified_by_row)
+
+        # Find and remove all column names referring to unmodified subsections, preserving others
+        columns = self.columns.copy()
+        for unmodified_column in self._subsection_column_names(unmodified_subsections, self.subsection_prefixes):
+            columns.remove(unmodified_column)
+
+        return columns
+
 
 class InterventionCSVProcessor(GradedSubsectionMixin, CSVProcessor):
     """
@@ -552,14 +604,14 @@ class InterventionCSVProcessor(GradedSubsectionMixin, CSVProcessor):
                 'course grade letter': course_grade.letter_grade,
                 'course grade numeric': course_grade.percent
             }
-            for block_id, (subsection, display_name) in iteritems(self._subsections):
-                row['name-{}'.format(block_id)] = display_name
+            for block_id, (subsection, display_name) in self._subsections.items():
+                row[f'name-{block_id}'] = display_name
                 grade = grades.get(subsection.location, None)
                 if grade:
                     if getattr(grade, 'override', None):
-                        row['grade-{}'.format(block_id)] = grade.override.earned_graded_override
+                        row[f'grade-{block_id}'] = grade.override.earned_graded_override
                     else:
-                        row['grade-{}'.format(block_id)] = grade.earned_graded
+                        row[f'grade-{block_id}'] = grade.earned_graded
             yield row
 
 
